/* libStreamedXML/src/lib/Core/Decoder.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace lsx {



const char* Decoder::sxmlRestartSequence = "<![RESTART[";



Decoder::Decoder(Parser* parser, utf8::Decoder* decoder)
    : parser(parser), decoder(decoder ?: new utf8::Decoder), sxmlRestartCount(0)
{
}



Decoder::~Decoder()
{
    delete decoder;
}



void Decoder::feedData(const char* bytes, size_t amount)
{
    /* We parse through the data once looking for restart sequences. If we encounter one, then we
    throw away all data leading up to it and continue from just after the marker. This can happen
    more than once per chunk of data, and can also happen over chunk boundaries. */
restart:
    for(size_t pos = 0; pos < amount; ++pos) {
        if(bytes[pos] == sxmlRestartSequence[sxmlRestartCount]) {
            if(++sxmlRestartCount == 11) {
                parser->streamRestart();
                decoder->reset();
                bytes += pos;
                amount -= pos;
                goto restart;
            }
        } else {
            sxmlRestartCount = 0;
        }
    }
    
    decoder->decoded.clear();
    decoder->decode(bytes, amount);
    parser->feedCharData(decoder->decoded);
}



void Decoder::restart()
{
    decoder->reset();
    sxmlRestartCount = 0;
}



}
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
